import threading, time
from gi.repository import Gtk, GObject, Gdk

from pywinusb.creator import USBCreator
from pywinusb.progress import ProgressWindow

# Okno aplikacji
class AppWindow(USBCreator, Gtk.Window):
    def __init__(self):
        super().__init__()
        Gtk.Window.__init__(self, title="pyWinUSB")

        self.set_border_width(10)
        self.set_size_request(150, -1)
        self.connect("delete-event", Gtk.main_quit)
        self.__create_wizard()

    def __create_wizard(self):
        """ Tworzenie kontrolek w oknie aplikacji
        """
        box = Gtk.Box(spacing=16)
        listbox = Gtk.ListBox()
        listbox.set_selection_mode(Gtk.SelectionMode.NONE)
        box.pack_start(listbox, True, True, 0)
        self.add(box)

        # Ścieżka do obrazu
        row = Gtk.ListBoxRow()
        box = Gtk.HBox(spacing=8)

        row.add(box)
        box.pack_start(Gtk.Label("Image:"), False, False, 0)
        self.path = Gtk.Entry(xalign=0)
        box.pack_start(self.path, True, True, 0)

        open_button = Gtk.Button(stock=Gtk.STOCK_OPEN)
        open_button.connect("clicked", self.__show_file_chooser)
        box.pack_start(open_button, False, False, 0)

        listbox.add(row)

        # Dostępne urządzenia
        row = Gtk.ListBoxRow()
        box = Gtk.VBox(spacing=8)
        box.pack_start(Gtk.Label("Select device:", xalign=0), True, False, 0)

        # Listowanie urządzeń co pół sekundy
        self.devices = Gtk.TreeView(Gtk.ListStore(str, str))
        self.devices.set_size_request(-1, 128)
        self.devices.append_column(Gtk.TreeViewColumn("", Gtk.CellRendererText(), text=0))
        self.devices.append_column(Gtk.TreeViewColumn("", Gtk.CellRendererText(), text=1))
        self.devices.set_headers_visible(False)
        box.pack_start(self.devices, True, False, 0)

        row.add(box)
        listbox.add(row)

        # Checkbox formatowania
        row = Gtk.ListBoxRow()
        box = Gtk.VBox(spacing=8)

        self.erase_checkbox = Gtk.CheckButton(label="Format device to NTFS (recommended)")
        self.erase_checkbox.set_active(True)
        box.pack_start(self.erase_checkbox, True, True, 0)

        row.add(box)
        listbox.add(row)

        # Podział
        listbox.add(Gtk.HSeparator())

        # Przyciski nawigacji
        row = Gtk.ListBoxRow()
        box = Gtk.HBox(spacing=4)
        box.pack_start( Gtk.LinkButton("http://www.github.com/Mati365", "Visit github repository", xalign=0)
                      , True, True, 0)

        apply = Gtk.Button(stock=Gtk.STOCK_APPLY)
        apply.connect("clicked", self.__create_boot_disc)
        box.pack_start(apply, True, True, 0)
        row.add(box)
        listbox.add(row)

        # Ładowanie listy urządzeń
        self.__load_device_list()

    def __create_boot_disc(self, button):
        """ Tworzenie bootowoalnego pendrive
        """
        path = self.path.get_text()
        if not path:
            raise Exception('Image path is empty')

        # Pobieranie z zaznaczenia
        (model, iter) = self.devices.get_selection().get_selected()
        if iter is None:
            raise Exception('Device is empty')

        # Pokazywanie okienka
        self.event_handler = ProgressWindow(self)
        self.event_handler.show_all()

        # Tworzenie dysku uruchomieniowego
        self.create_boot_disc(
              device=model[iter][0]
            , image_path=path
            , format_device=self.erase_checkbox.get_active()
        )

    def __show_file_chooser(self, button):
        """ Tworzenie dialogu wyboru pliku obrazu
        """
        dialog = Gtk.FileChooserDialog("Please choose a file", self, Gtk.FileChooserAction.OPEN, (
              Gtk.STOCK_CANCEL
            , Gtk.ResponseType.CANCEL
            , Gtk.STOCK_OPEN
            , Gtk.ResponseType.OK
        ))

        filters = {
            "application/x-cd-image": "CD/DVD image"
        }
        for key, val in filters.items():
            filter_text = Gtk.FileFilter()
            filter_text.set_name(val)
            filter_text.add_mime_type(key)
            dialog.add_filter(filter_text)

        response = dialog.run()
        if response == Gtk.ResponseType.OK:
            self.path.set_text(dialog.get_filename())
        elif response == Gtk.ResponseType.CANCEL:
            print("Cancel clicked")
        dialog.destroy()

    __cached_devices = []  # Urządzenia aktualnie widoczne w liście urządzeń
    def __load_device_list(self):
        """ Wczytywanie nowej listy urządzeń do listy
        co sekundę i kasowanie poprzedniej
        """
        def update_list():
            while True:
                # Wczytywanie nowej listy, jeśli znajdzie nowe urządzenie to wczytywanie od nowa
                new_devices = USBCreator.list_devices()
                if self.__cached_devices != new_devices:
                    self.__cached_devices = new_devices

                    # Wczytywanie nowych urządzeń do kontrolki
                    devices = self.devices.get_model()
                    devices.clear()

                    # Dodawanie urządzeń do listy
                    for (mount_point, size) in new_devices:
                        devices.append([mount_point, str(size/1048576)+"MB"])
                time.sleep(1)

        # Włączanie wątku odpowiedzialnego za odświeżanie urządzeń
        threading.Thread(target=update_list, daemon=True).start()



import re

from gi.repository import Gtk, GObject
from pywinusb.events import EventHandler
from pywinusb.creator import MessageBox

# Okno kopiowania plików
class ProgressWindow(Gtk.Window, EventHandler):
    def __init__(self, creator):
        Gtk.Window.__init__(self, type=Gtk.WindowType.TOPLEVEL, title="File copying")

        self.creator = creator

        self.connect("delete-event", lambda *args, **kwargs: True)
        self.connect("destroy-event", lambda *args, **kwargs: True)

        self.set_border_width(10)
        self.set_size_request(400, -1)
        self.set_modal(True)

        self.__create_window()

    def __create_window(self):
        box = Gtk.VBox(spacing=8)
        self.add(box)

        # Pasek postępu
        self.progress_bar = Gtk.ProgressBar()
        self.progress_bar.set_show_text(True)
        box.pack_start(self.progress_bar, True, True, 0)

        # Pasek statusu
        self.status_bar = Gtk.Statusbar()
        self.status_context = self.status_bar.get_context_id("Status")
        box.pack_start(self.status_bar, True, True, 0)

    def update_progress(self, file, percent):
        """ Aktualizacja paska procentu
        :param file:    Plik
        :param percent: Procent
        """
        self.progress_bar.set_fraction(percent)
        self.progress_bar.set_text("File: {file} - {percent}%".format(file=file, percent=int(percent * 100)))

    def update_status(self, status):
        """ Aktualizacja pasku statusu
        :param status: Treść statusu
        """
        self.status_bar.push(self.status_context, status)

    def show_warning(self, title, content, type=Gtk.MessageType.INFO):
        """
        :param title:   Tytuł
        :param content: Treść
        :return:
        """
        MessageBox(self.creator, title, content, Gtk.ButtonsType.OK, type)

    # Callbacki z innego wątku
    def on_status(self, status):
        GObject.idle_add(self.update_status, status)
    def on_progress(self, total, current, file):
        GObject.idle_add( self.update_progress
                        , re.search("pyWinUSB\/\w{40}\/source\/(.*)", file).group(1)
                        , current / total
                        )
    def on_done(self, err=None):
        if err is not None:
            GObject.idle_add(self.show_warning, "Error!", err, Gtk.MessageType.WARNING)
        else:
            GObject.idle_add(self.show_warning, "Copying done!", "You can eject device!")
        self.destroy()


import os, sys
from gi.repository import Gtk, GObject, Gdk
from pywinusb.window import AppWindow

def check_root_access():
    """ Sprwdzenie czy skrypt jest instalowany pod rootem
    :return: True jeśli root
    """
    return not os.geteuid()

def main():
    if not check_root_access():
        sys.exit("\nOnly root can run this script :(\n")

    win = AppWindow()
    win.show_all()

    Gdk.threads_enter()
    GObject.threads_init()
    Gtk.main()
    Gdk.threads_leave()

if __name__ == '__main__':
    sys.exit(main() or 0)


import sh, os, re, threading, hashlib, subprocess
from gi.repository import Gtk

from pywinusb.decorators import chain_method, event_method
from pywinusb.events import EventHandler
from pywinusb.decorators import installer_method

# Dialog potwierdzenia
def MessageBox(parent, title, message, buttons=Gtk.ButtonsType.OK_CANCEL, type=Gtk.MessageType.WARNING):
    dialog = Gtk.MessageDialog(parent, 0, type, buttons, title)
    dialog.format_secondary_text(message)
    response = dialog.run()
    dialog.destroy()
    return response

# Klasa kreatora dysku
class USBCreator:
    def __init__(self, event_handler = EventHandler()):
        self.source_mount = None
        self.destination_mount = None
        self.copy_thread = None
        self.event_handler = event_handler

        self.format_filesystem = "ntfs"
        self.format_table = "msdos"

    @staticmethod
    def get_filesystem_name(device, partition=1):
        """
        :param device:      Urządzenie
        :param partition:   Numer partycji
        :return: System plików na partycji
        """
        return re.search("\n\w*.(\w*)", str(sh.lsblk(device + str(partition), fs=True))).group(1)

    @staticmethod
    def get_device_size(device, whole_device=False):
        """ Pobieranie rozmiarów urządzenia/partycji
        :param already_formatted: Czy jest właśnie sformatowany
        :return: Rozmiar w bajtach
        """
        try:
            if not whole_device:
                # Sprawdzanie systemu plików, jeśli nie jest to NTFS to wysypuje się
                return int(re.search("\n\/dev\/(?:\w*\s*){3}(\d*)", str(sh.df(device + "1"))).group(1)) * 1024

            # Pobieranie rozmiaru całego urządzenia
            return int(sh.blockdev("--getsize64", device))
        except:
            return -1

    @staticmethod
    def list_devices():
        """ Listowanie wszystkich punktów montowania wszystkich pendrive
        :return: Lista urządzeń
        """
        mount_points = []
        popen = os.popen("grep -lir '1' /sys/block/sd*/removable")
        for block_device in iter(popen.readline, ""):
            device = "/dev/{}".format(re.search("(\w*)\/removable$", block_device).group(1))
            mount_points.append([device, USBCreator.get_device_size(device, True)])
        return mount_points

    @staticmethod
    def get_mount_path(device):
        """ Pobieranie ścieżki, w której zamontowane jest urządzenie
        :param device: Urządzenie np. /dev/sdb
        """
        for line in sh.mount():
            match = re.search("^{}.*on.(.*) type".format(device), line)
            if match is not None:
                return match.group(1)

    @installer_method("Opening device...")
    def open_device(self):
        # Odmontowywanie urządzenia jeśli jest ono zamontowane
        mount_path = USBCreator.get_mount_path(self.device)
        if mount_path:
            response = MessageBox( parent=self
                                 , title="Device is already in use"
                                 , message="Unmount device {device} mounted in:\n {mount_path}?"
                                    .format(device=self.device, mount_path=mount_path)
                                 )
            if response == Gtk.ResponseType.OK:
                # Odmontowywanie urządzenia jeśli zatwierdzone
                sh.umount(mount_path)
            else:
                raise Exception("Device must be unmounted before copying!")

    @installer_method("Erasing device...")
    def erase_device(self):
        """ Wymazywanie całego urządzenia
        """
        # Tworzenie nowej tablicy partycji
        subprocess.call(["parted", self.device, "mklabel", "msdos", "--script"])
        subprocess.call(["parted", self.device, "mkpart", "primary", "ntfs", "0%", "100%"])

        # Formatowanie na NTFSa
        subprocess.call(["mkfs.ntfs", "-Q", self.device + "1"])

    @installer_method("Mounting devices...")
    def mount_devices(self):
        """ Montowanie obrazu dysku
        :param image_path: Ścieżka bezwzględna do obrazu
        """
        self.mount_folder = "/media/pyWinUSB/{}".format(hashlib.sha1(self.image_path.encode("utf-8")).hexdigest())
        self.source_mount = self.mount_folder  + "/source"
        self.destination_mount = self.mount_folder  + "/destination"

        # Odmontowywanie na wszelki wypadek
        try: sh.umount(self.source_mount)
        except:
            pass

        # Montownie obrazu ISO dysku
        sh.mkdir(self.source_mount, self.destination_mount, "-p")
        sh.mount(self.image_path, self.source_mount, o="loop")

        # Montowanie urządzenia
        subprocess.call(["sfdisk", self.device, "-R"])
        subprocess.call(["mount", self.device + "1", self.destination_mount])

        # Sprawdzanie systemu plików pod kątem rozmiaru
        if USBCreator.get_device_size(self.device) < os.path.getsize(self.image_path):
            raise Exception("No enough space on disk/partition!")

        # Sprawdzenie typu systemu plików
        # elif USBCreator.get_filesystem_name(self.device) != "ntfs":
        #     raise Exception("Unsupported filesystem!")


    @installer_method("Copying files...")
    def copy_files(self):
        # Pliki do skopiowania
        files_to_copy = []
        for root, dirnames, filenames in os.walk(self.source_mount):
            for file in filenames:
                files_to_copy.append(root + '/' + file)

        # Modyfikacja ścieżki oraz kopiowanie plików
        for index, file in enumerate(files_to_copy):
            match = re.search("pyWinUSB\/\w{40}\/source\/(.*\/)", file)
            dest_file = "/".join([
                  self.destination_mount
                , match.group(1) if match else ""
            ])

            # Tworzenie i kopiowanie
            self.event_handler.on_progress(len(files_to_copy), index, file)
            sh.mkdir(dest_file, "-p")
            sh.cp(file, dest_file)

    @installer_method("Installing GRUB...")
    def make_bootable(self):
        """ Tworzenie dysku bootowalnego
        """
        # self.uuid = re.search("UUID=\"(\w*)\"", str(sh.blkid(self.device + "1"))).group(1)
        # print("Device UUID:", self.uuid)

        # W niektórych wersjach windows katalog ten jest z drukowanej
        def try_move(old_file, new_file):
            try: sh.mv(old_file, new_file)
            except:
                print("File {} already exists, nothing to move".format(new_file))

        self.boot_folder = self.destination_mount + "/boot"
        try_move(self.destination_mount + "/BOOT", self.boot_folder)
        try_move(self.destination_mount + "/BOOTMGR", self.destination_mount + "/bootmgr")

        # Instalownie bootloadera
        # grub-install --target=i386-pc --boot-directory="/<USB_mount_folder>/boot" /dev/sdX
        installer = sh.Command(sh.which("grub-install")
                               or sh.which("grub2-install"))
        installer(self.device, target="i386-pc", skip_fs_probe=True, force=True, boot_directory=self.destination_mount + "/boot")

        # Tworzenie konfiguracji GRUBa
        with open( "{}/{}/grub.cfg".format( self.boot_folder, "grub2" if str(installer).find("grub2") != -1 else "grub")
                 , "wt"
                 ) as config:
            config.write("""
                set menu_color_normal=white/black
                set menu_color_highlight=black/light-gray
                menuentry 'Install Windows' {
                    ntldr /bootmgr
                }
            """)

    @installer_method("Umount devices...")
    def close_stream(self):
        """ Zamykanie urządzenia i kasowanie plików tymczasowych
        """
        if self.source_mount is not None:
            sh.sync()
            sh.umount(self.source_mount, self.destination_mount)
            sh.rm(self.mount_folder, '-rf')

    def create_boot_disc(self, device, image_path, format_device=False):
        """ Tworzenie dysku instalacyjnego na pendrive
        :param device:      Urządzenie np. /dev/sdb
        :param image_path:  Ścieżka do surowego obrazu
        """
        self.device = device
        self.image_path = image_path

        # Instalacja w innym wątku by nie wieszać UI
        try:
            err_title = None
            self.open_device()
            if format_device:
                self.erase_device()
            self.mount_devices()

            # Wątek kopiowania
            def worker():
                self \
                    .copy_files() \
                    .make_bootable() \
                    .close_stream() \
                    .event_handler.on_done()
            self.copy_thread = threading.Thread(target=worker, daemon=True)
            self.copy_thread.start()

        # Po złapaniu błędu wyświetlany jest warning i odmontowywane urządzenia
        except Exception as err:
            err_title = err.args[0]
        except:
            print(os.sys.exc_info())
            err_title = os.sys.exc_info()[0]
        finally:
            if err_title is not None:
                self\
                    .close_stream()\
                    .event_handler.on_done(err_title)

    def stop_copying(self):
        """ Zatrzymywania kopiowania plików np. po zamknięciu dialogu
        """
        self.close_stream()
        if self.copy_thread is not None:
            self.copy_thread.quit()


#!/usr/bin/env python3
from setuptools import setup
from pip.req import parse_requirements

install_reqs = parse_requirements("requirements.txt", session=False)
setup(
      name="py-winusb"
    , version="0.2.0.1"
    , packages=["pywinusb"]
    , install_requires= [str(ir.req) for ir in install_reqs]
    , url="https://github.com/Mati365/pyWinUSB"
    , license="MIT"
    , author="Mateusz Bagiński"
    , author_email="cziken58@gmail.com"
    , description="Tool that helps with creating bootable Windows USB drives"
    , long_description=open("README.rst").read()
    , zip_safe=False
    , entry_points={
        "console_scripts": ["pywinusb=pywinusb.__main__:main"]
    }
    , platforms="linux"
    , keywords=["windows", "usb", "installer"]
    , package_data={"pywinusb": ["requirements.txt"]}
    , include_package_data=True
    , classifiers=[
          "Development Status :: 2 - Pre-Alpha"
        , "Intended Audience :: Developers"
        , "Intended Audience :: Information Technology"
        , "Programming Language :: Python"
        , "Programming Language :: Python :: 3"
        , "Programming Language :: Python :: 3.3"
        , "Programming Language :: Python :: 3.4"
    ]
)



# Dla fluent API
def chain_method(func):
    def func_wrapper(self, *args, **kwargs):
        func(self, *args, **kwargs)
        return self
    return func_wrapper

# Szybkie emitowanie zdarzeń
def event_method(status):
    def dec(func):
        def func_wrapper(self, *args, **kwargs):
            self.event_handler.on_status(status)
            func(self, *args, **kwargs)
            return self
        return func_wrapper
    return dec

# Metoda instalacyjna
def installer_method(status):
    def dec(func):
        return chain_method(event_method(status)(func))
    return dec


# Klasa odpowiedzialna za eventy
class EventHandler:
    def on_progress(self, total, current, file):
        """ Metoda wywoływana podczas kopiowania pliku
        :param total:   Liczba wszystkich plików
        :param current: Aktualny index pliku
        :param file:    Plik
        :return:
        """
        pass

    def on_status(self, status):
        """ Metoda wywoływna podczas zmiany statusu
        :param status: Informacja na statusbar
        :return:
        """
        pass

    def on_done(self, err):
        """ Metoda wywoływana po skończeniu kopiowania
        :param err: True jeśli błąd
        """
        pass